﻿Imports System.Data.Common
Imports System.Data
Imports System.Windows.Forms
Imports System.ComponentModel
Imports System.Runtime.Serialization

#If Not NET_FRAMEWORK_V2 Then
<DataContract()> _
Public Class ItemWithID
#Else
  Public Class ItemWithID
#End If

  Implements INotifyPropertyChanged


  'Private m_sValue As String = ""
  Dim m_sName As String = ""

#If Not NET_FRAMEWORK_V2 Then
  <DataMember()> _
  Public Property Name As String
#Else
    Public Property Name As String
#End If

    Get
      Return m_sName
    End Get
    Set(value As String)
      m_sName = value
      RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Name"))
    End Set
  End Property
  Dim m_sID As String = ""

#If Not NET_FRAMEWORK_V2 Then
  <DataMember()> _
  Public Property ID As String
#Else
    Public Property ID As String
#End If
    Get
      Return m_sID
    End Get
    Set(value As String)
      m_sID = value
      RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("ID"))
    End Set
  End Property

  Public Sub New()
    Name = ""
    ID = -1
  End Sub
  Public Sub New(ByVal sValue As String, ByVal sID As String)
    Name = sValue
    ID = sID
  End Sub

  Public Overrides Function ToString() As String
    If Name Is Nothing Then Return ""
    Return Name
  End Function

  Public Shared Operator =(ByVal item1 As ItemWithID, ByVal item2 As ItemWithID) As Boolean
    Return item1.ID = item2.ID
  End Operator

  Public Shared Operator <>(ByVal item1 As ItemWithID, ByVal item2 As ItemWithID) As Boolean
    Return Not (item1 = item2)
  End Operator

  Public Shared Function GetListIdIndex(iID As ItemWithID, lst As System.Windows.Forms.ListBox) As Integer
    Dim iRet As Integer = -1, iListID As ItemWithID
    For i As Integer = 0 To lst.Items.Count - 1
      iListID = lst.Items(i)
      If iID.ID = iListID.ID Then
        iRet = i
        Exit For
      End If
    Next
    Return iRet
  End Function

  Public Shared Function GetListEntryIndex(iID As ItemWithID, lst As System.Windows.Forms.ListBox) As Integer
    Dim iRet As Integer = -1, iListID As ItemWithID
    For i As Integer = 0 To lst.Items.Count - 1
      iListID = lst.Items(i)
      If (iID.ID = iListID.ID) And (iID.Name = iListID.Name) Then
        iRet = i
        Exit For
      End If
    Next
    Return iRet
  End Function

  ''' <summary>
  ''' If returning to Integer, Set default if cbo may have selected index of -1
  ''' </summary>
  ''' <param name="cbo"></param>
  ''' <param name="oDefault"></param>
  ''' <returns></returns>
  ''' <remarks></remarks>
  Public Shared Function GetComboIDValue(ByVal cbo As System.Windows.Forms.ComboBox, Optional ByVal oDefault As Object = Nothing) As Object
    If (cbo.SelectedIndex >= 0) Then
      Return cbo.Items(cbo.SelectedIndex).ID
    Else
      If oDefault Is Nothing Then
        Return DBNull.Value
      Else
        Return oDefault
      End If
    End If
  End Function
  Public Shared Sub SetComboEntry(ByRef cbo As System.Windows.Forms.ComboBox, ByVal iItemWithID As Integer, Optional bIgnoreINVALID As Boolean = False)

    Dim iIndex As Integer = -1
    Try
      If bIgnoreINVALID Or iItemWithID <> INVALID_ID Then
        Dim iListID As ItemWithID
        For i As Integer = 0 To cbo.Items.Count - 1
          iListID = cbo.Items(i)
          If (iItemWithID = iListID.ID) Then
            iIndex = i
            Exit For
          End If
        Next
      End If

    Catch ex As Exception
      MsgBox(ex.Message)
    End Try

    Try
      'cbo.Select() ' needed so object can be accessed
      cbo.SelectedIndex = iIndex
    Catch ex As Exception
    End Try

  End Sub

  Public Event PropertyChanged(sender As Object, e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged
End Class

Public Class ItemWithIDAndType
  Inherits ItemWithID
  Public Type As String
  Public Sub New(ByVal sValue As String, ByVal sID As String, ByVal sType As String)
    MyBase.New(sValue, sID)
    Type = sType
  End Sub
  Public Overrides Function ToString() As String
    Return MyBase.ToString() & ": " & Type
  End Function
End Class

Public Class ItemWithIDAndRequired
  Inherits ItemWithID
  Public Required As Boolean
  Public Sub New(ByVal sValue As String, ByVal sID As String, ByVal bRequired As Boolean)
    MyBase.New(sValue, sID)
    Required = bRequired
  End Sub
  Public Overrides Function ToString() As String
    Return MyBase.ToString & IIf(Required, " - Required", " - Not Required")
  End Function

End Class

Public Class ItemWithIDAndDataRow
  Inherits ItemWithID
  Public DRData As DataRow
  Public Sub New(ByVal sValue As String, ByVal iID As Integer, ByVal DR As DataRow)
    MyBase.New(sValue, iID)
    DRData = DR
  End Sub
End Class
Public Class ListItemWithID
  Inherits ListViewItem
  Public Value As String = ""
  Public ID As String = ""
  Public Sub New()
    Value = ""
    ID = -1
  End Sub
  Public Sub New(ByVal sValue As String, ByVal sID As String)
    Value = sValue
    ID = sID
  End Sub

  Public Overrides Function ToString() As String
    Return Value
  End Function
End Class
