﻿Imports System.Reflection
Imports System.Text
Imports XPressEntry
Imports XPressEntrySDKServiceInterface
Public Class frmSDKTestClient
  Private Sub lstInfo_SelectedIndexChanged(sender As Object, e As EventArgs) Handles lstInfo.SelectedIndexChanged
    Dim iwi As ItemWithIDAndMessage = lstInfo.SelectedItem
    If iwi IsNot Nothing Then
      txtDetails.Text = iwi.FullMessage
    Else
      txtDetails.Text = ""
    End If

  End Sub
  Private g_cXPESDKService As XPressEntrySDKServiceInterface.XPressEntrySDKServiceInterface

  Public Sub CreateOrUpdateXPESDKServiceObject(sMachineName As String, sUsername As String, sPassword As String, bValidateCert As Boolean, sCertID As String)
    If (g_cXPESDKService Is Nothing) Then
      g_cXPESDKService = New XPressEntrySDKServiceInterface.XPressEntrySDKServiceInterface(sMachineName, sUsername, sPassword, "SDK Test Application", bValidateCert, sCertID)
      AddHandler g_cXPESDKService.ActivityCreated, AddressOf ActivityCreatedHandler
      AddHandler g_cXPESDKService.DataObjectsChanged, AddressOf DataObjectsChangedHandler
      AddHandler g_cXPESDKService.DataObjectsDeleted, AddressOf DataObjectsDeletedHandler
      AddHandler g_cXPESDKService.OnServiceStatusChanged, AddressOf ServiceStatusChangedHandler

    Else
      g_cXPESDKService.SetServiceConnectionDetails(sMachineName, sUsername, sPassword, bValidateCert, sCertID)
    End If
  End Sub

  Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
    m_lstDataTypes = New List(Of XPEDataTypes)([Enum].GetValues(GetType(XPEDataTypes)))
    cboSetNextTest.Items.Clear()
    For Each d As XPEDataTypes In m_lstDataTypes
      cboSetNextTest.Items.Add(New ItemWithID(d.ToString, CInt(d)))
    Next
    cboSetNextTest.SelectedIndex = 0

    CreateOrUpdateXPESDKServiceObject(txtServiceMachine.Text, txtServiceUsername.Text, txtServicePassword.Text, False, "XPressEntryService")
  End Sub



  Private Sub ServiceStatusChangedHandler(bConnected As Boolean, sErr As String)
    Dim sMsg As String = Now.ToLongTimeString & " : " & "Service Status : " & IIf(bConnected, "CONNECTED", "DISCONNECTED")
    AddEventListItem(sMsg, sMsg)
  End Sub

  Private Sub DataObjectsDeletedHandler(lst As List(Of BaseData))
    Dim t1 As Type = Nothing
    If lst.Count > 0 Then
      t1 = lst(0).GetType()
      AddEventListItem(lst.Count & " Objects Deleted of " & t1.FullName, lst.Count & " Objects Deleted")
    End If

  End Sub

  Private Sub DataObjectsChangedHandler(lst As List(Of BaseData))
    Dim t1 As Type = Nothing
    If lst.Count > 0 Then
      t1 = lst(0).GetType()
      AddEventListItem(lst.Count & " Objects Changed of " & t1.FullName, lst.Count & " Objects Changed")
    End If
  End Sub

  Private Sub ActivityCreatedHandler(lst As List(Of ActivityData))
    Dim t1 As Type = Nothing
    If lst.Count > 0 Then
      t1 = lst(0).GetType()
      AddEventListItem(lst.Count & " Activities Changed of " & t1.FullName, lst.Count & " Activities Changed")
    End If
  End Sub

  Private Sub AddInfoListItem(sTitle As String, sFullMessage As String)
    lstInfo.Items.Add(New ItemWithIDAndMessage(sTitle, "", sFullMessage))
    txtDetails.Text = sFullMessage
  End Sub

  Delegate Sub AddEventListItemDelegate(sTitle As String, sFullMessage As String)
  Private Sub AddEventListItem(sTitle As String, sFullMessage As String)
    If Me.InvokeRequired Then
      Me.BeginInvoke(New AddEventListItemDelegate(AddressOf AddEventListItem), sTitle, sFullMessage)
    Else
      lstEvents.Items.Add(New ItemWithIDAndMessage(sTitle, "", sFullMessage))
    End If

  End Sub
  Private Class ItemWithIDAndMessage
    Inherits ItemWithID
    Private m_sMessage As String = ""
    Public ReadOnly Property FullMessage
      Get
        Return m_sMessage
      End Get
    End Property
    Public Sub New(sName As String, sID As String, sFullMessage As String)
      MyBase.New(sName, sID)
      m_sMessage = sFullMessage
    End Sub

  End Class

  Private Sub btnConnect_Click(sender As Object, e As EventArgs) Handles btnConnect.Click
    CreateOrUpdateXPESDKServiceObject(txtServiceMachine.Text, txtServiceUsername.Text, txtServicePassword.Text, False, "XPressEntryService")
    Try
      g_cXPESDKService.SilentPing()
      AddInfoListItem("Connect Success", "")
    Catch ex As Exception
      AddInfoListItem("Connect Failed", GetExceptionText(ex))
    End Try
  End Sub

  Private Sub btnSubscribe_Click(sender As Object, e As EventArgs) Handles btnSubscribe.Click
    CreateOrUpdateXPESDKServiceObject(txtServiceMachine.Text, txtServiceUsername.Text, txtServicePassword.Text, False, "XPressEntryService")
    Try
      g_cXPESDKService.SubscribeClient("SDK Test")
      AddEventListItem("Client Subscribed", "")
    Catch ex As Exception
      AddEventListItem("Client Subscription Failed", GetExceptionText(ex))
    End Try

  End Sub

  Private Sub cboSetNextTest_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboSetNextTest.SelectedIndexChanged
    btnTestNext.Text = "Test " & m_lstDataTypes(cboSetNextTest.SelectedIndex).ToString()
  End Sub

  Private Function PrintObjectInfo(o As Object) As String
    Dim sbRet As New StringBuilder
    Dim T1 As Type = o.GetType()
    Dim props() As PropertyInfo = T1.GetProperties()
    sbRet.AppendLine("Object Type:" & T1.Name)
    For Each p As PropertyInfo In props
      Try
        sbRet.AppendLine(p.Name & ":" & p.GetValue(o).ToString())
      Catch ex As Exception
        sbRet.AppendLine("Error with : " & p.Name & " : " & ex.Message)
      End Try
    Next

    Return sbRet.ToString
  End Function

  Dim m_lstDataTypes As List(Of XPEDataTypes)
  Private Function TestSingleDataType(d As XPEDataTypes) As Boolean
    Dim bRet As Boolean = False
    Dim sErr As String = ""
    Try
      Dim T1 As Type = BaseData.GetTypeFromXPEDataType(d)
      'This is a GET call
      Dim lst As List(Of BaseData) = g_cXPESDKService.FindDataObjects(d, "", -1, -1, "", sErr)
      If lst Is Nothing Then
        AddInfoListItem("Error: " & sErr, sErr)
        bRet = False
      Else
        AddInfoListItem(lst.Count & " found of type " & d.ToString(), sErr)
        bRet = True
      End If

      'THIS SHOULD BE AN UPDATE call
      If bRet Then

        If lst.Count = 0 Then
          AddInfoListItem("Nothing to Update " & T1.Name, sErr)
        Else
          Dim o As Object = lst(0)
          If d = XPEDataTypes.UserData Then
            Dim u As UserData = o
            u.first_name = "Updated" & Now.ToShortTimeString
            u.last_name = "Updated" & Now.ToShortTimeString


            o = u
          ElseIf d = XPEDataTypes.VisitorData Then
            Dim u As VisitorData = o
            u.first_name = "Updated" & Now.ToShortTimeString
            u.last_name = "Updated" & Now.ToShortTimeString


            o = u

          Else
            Dim fields() As FieldInfo = T1.GetFields
            For Each p As FieldInfo In fields
              If p.Name = "deleted_at" Then Continue For
              If p.FieldType Is GetType(String) Then
                If p.Name <> "external_id" Then
                  p.SetValue(o, "Updated" & T1.Name & " " & p.Name)
                End If
              ElseIf p.FieldType Is GetType(Integer) Then
                'Don't update any ID fields
                If p.Name.Contains("id") Then
                  p.SetValue(o, 1)
                End If
              ElseIf p.FieldType Is GetType(Boolean) Then
                p.SetValue(o, True)
              ElseIf p.FieldType Is GetType(DateTime) Then
                p.SetValue(o, Now.ToUniversalTime)
              End If
            Next

          End If

          If g_cXPESDKService.UpdateOrCreateDataObject(o, sErr) Then
            AddInfoListItem("Success Updating " & T1.Name, PrintObjectInfo(o))
          Else
            AddInfoListItem("Error Updating " & T1.Name, PrintObjectInfo(o))
          End If
        End If

      End If

      'THIS SHOULD BE A CREATE call
      Dim oCreate As Object = Activator.CreateInstance(T1)
      If bRet Then
        If d = XPEDataTypes.UserData Then
          Dim u As UserData = oCreate
          u.first_name = "Created" & Now.ToShortTimeString
          u.last_name = "Created" & Now.ToShortTimeString


          oCreate = u
        ElseIf d = XPEDataTypes.VisitorData Then
          Dim u As VisitorData = oCreate
          u.first_name = "VisitorCreated" & Now.ToShortTimeString
          u.last_name = "VisitorCreated" & Now.ToShortTimeString


          oCreate = u

        Else
          Dim fields() As FieldInfo = T1.GetFields
        For Each p As FieldInfo In fields
          If p.Name = "deleted_at" Then Continue For
          If p.Name.ToLower = "id" Then Continue For
          If p.FieldType Is GetType(String) Then
            p.SetValue(oCreate, T1.Name & " " & p.Name)
          ElseIf p.FieldType Is GetType(Integer) Then
            If p.Name.Contains("id") Then
              p.SetValue(oCreate, 1)
            End If
          ElseIf p.FieldType Is GetType(Boolean) Then
            p.SetValue(oCreate, True)
          ElseIf p.FieldType Is GetType(DateTime) Then
            p.SetValue(oCreate, Now.ToUniversalTime)
          End If
        Next
        End If
        If g_cXPESDKService.UpdateOrCreateDataObject(oCreate, sErr) Then
          AddInfoListItem("Success Creating  " & T1.Name, PrintObjectInfo(oCreate))
        Else
          AddInfoListItem("Error Creating " & T1.Name, sErr)
        End If
      End If

      'THIS SHOULD BE A DELETE call
      If bRet Then

      End If

    Catch ex As Exception
      AddInfoListItem("Error Testing " & d.ToString(), GetExceptionText(ex))
      bRet = False
    End Try


    Return bRet
  End Function
  Private Sub btnTestNext_Click(sender As Object, e As EventArgs) Handles btnTestNext.Click
    Dim sErr As String = ""
    If TestSingleDataType(m_lstDataTypes(cboSetNextTest.SelectedIndex)) Then
      If cboSetNextTest.SelectedIndex + 1 >= cboSetNextTest.Items.Count Then
        cboSetNextTest.SelectedIndex = 0
      Else
        cboSetNextTest.SelectedIndex += 1
      End If
    Else
      'Do nothing?
    End If

  End Sub


  Private Sub btnTestAll_Click(sender As Object, e As EventArgs) Handles btnTestAll.Click

    'Test them all and stop on an error
    For Each d As XPEDataTypes In m_lstDataTypes
      If Not TestSingleDataType(d) Then
        Exit For
      End If
    Next
  End Sub

End Class
